# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.defects_response import DefectsResponse
from ..types.drivers_vehicle_assignments_get_drivers_vehicle_assignments_response_body import (
    DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody,
)
from ..types.dvirs_list_response import DvirsListResponse
from ..types.idling_reports_get_vehicle_idling_reports_response_body import (
    IdlingReportsGetVehicleIdlingReportsResponseBody,
)
from ..types.safety_events_get_safety_activity_event_feed_response_body import (
    SafetyEventsGetSafetyActivityEventFeedResponseBody,
)
from ..types.safety_events_list_response import SafetyEventsListResponse
from ..types.v_1_vehicle_harsh_event_response import V1VehicleHarshEventResponse
from ..types.vehicles_driver_assignments_get_vehicles_driver_assignments_response_body import (
    VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody,
)
from .types.get_drivers_vehicle_assignments_request_driver_activation_status import (
    GetDriversVehicleAssignmentsRequestDriverActivationStatus,
)


class RawLegacyApIsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_dvir_defects(
        self,
        *,
        start_time: str,
        end_time: str,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        is_resolved: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DefectsResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/streamdefects) instead. The endpoint will continue to function as documented.**

        Returns a list of DVIR defects in an organization, filtered by creation time. The maximum time period you can query for is 30 days.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Defects** under the Maintenance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). *The maximum time period you can query for is 30 days.*

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). *The maximum time period you can query for is 30 days.*

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        is_resolved : typing.Optional[bool]
            A filter on the data based on resolution status. Example: `isResolved=true`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DefectsResponse]
            List of all DVIR defects in the organization
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/defects/history",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "isResolved": is_resolved,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DefectsResponse,
                    parse_obj_as(
                        type_=DefectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_drivers_vehicle_assignments(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        driver_activation_status: typing.Optional[GetDriversVehicleAssignmentsRequestDriverActivationStatus] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getdrivervehicleassignments) instead. The endpoint will continue to function as documented.** Get all vehicle assignments for the requested drivers in the requested time range. The only type of assignment supported right now are assignments created through the driver app.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_activation_status : typing.Optional[GetDriversVehicleAssignmentsRequestDriverActivationStatus]
            If value is `deactivated`, only drivers that are deactivated will appear in the response. This parameter will default to `active` if not provided (fetching only active drivers).  Valid values: `active`, `deactivated`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/drivers/vehicle-assignments",
            method="GET",
            params={
                "driverIds": driver_ids,
                "startTime": start_time,
                "endTime": end_time,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverActivationStatus": driver_activation_status,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody,
                    parse_obj_as(
                        type_=DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_dvir_history(
        self,
        *,
        start_time: str,
        end_time: str,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DvirsListResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getdvirs) instead. The endpoint will continue to function as documented.**

         Returns a list of all DVIRs in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read DVIRs** under the Maintenance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DvirsListResponse]
            List of all DVIRs in the organization
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/dvirs/history",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "startTime": start_time,
                "endTime": end_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DvirsListResponse,
                    parse_obj_as(
                        type_=DvirsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_vehicle_idling_reports(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        is_pto_active: typing.Optional[bool] = None,
        min_idling_duration_minutes: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[IdlingReportsGetVehicleIdlingReportsResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getidlingevents) instead. The endpoint will continue to function as documented.** Get all vehicle idling reports for the requested time duration.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Fuel & Energy** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        is_pto_active : typing.Optional[bool]
            A filter on the data based on power take-off being active or inactive.

        min_idling_duration_minutes : typing.Optional[int]
            A filter on the data based on a minimum idling duration.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[IdlingReportsGetVehicleIdlingReportsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/reports/vehicle/idling",
            method="GET",
            params={
                "after": after,
                "limit": limit,
                "startTime": start_time,
                "endTime": end_time,
                "vehicleIds": vehicle_ids,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "isPtoActive": is_pto_active,
                "minIdlingDurationMinutes": min_idling_duration_minutes,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    IdlingReportsGetVehicleIdlingReportsResponseBody,
                    parse_obj_as(
                        type_=IdlingReportsGetVehicleIdlingReportsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_safety_events(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SafetyEventsListResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getsafetyeventsv2stream) instead. The endpoint will continue to function as documented.**

         Fetch safety events for the organization in a given time period.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of vehicle IDs. Example: `vehicleIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SafetyEventsListResponse]
            List of safety events from given time period.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/safety-events",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "vehicleIds": vehicle_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsListResponse,
                    parse_obj_as(
                        type_=SafetyEventsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_safety_activity_event_feed(
        self,
        *,
        after: typing.Optional[str] = None,
        start_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SafetyEventsGetSafetyActivityEventFeedResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getsafetyeventsv2stream) instead. The endpoint will continue to function as documented.**

        Get continuous safety events. The safety activity event feed offers a change-log for safety events. Use this endpoint to subscribe to safety event changes. See documentation below for all supported change-log types.

        | ActivityType      | Description |
        | ----------- | ----------- |
        | CreateSafetyEventActivityType | a new safety event is processed by Samsara      |
        | BehaviorLabelActivityType     | a label is added or removed from a safety event |
        | CoachingStateActivityType     | a safety event coaching state is updated        |

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SafetyEventsGetSafetyActivityEventFeedResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/safety-events/audit-logs/feed",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsGetSafetyActivityEventFeedResponseBody,
                    parse_obj_as(
                        type_=SafetyEventsGetSafetyActivityEventFeedResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_vehicles_driver_assignments(
        self,
        *,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getdrivervehicleassignments) instead. The endpoint will continue to function as documented.** Get all driver assignments for the requested vehicles in the requested time range. The only type of assignment supported right now are assignments created through the driver app.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/vehicles/driver-assignments",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "vehicleIds": vehicle_ids,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody,
                    parse_obj_as(
                        type_=VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_vehicle_harsh_event(
        self, vehicle_id: int, *, timestamp: int, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1VehicleHarshEventResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getsafetyeventsv2stream) instead. The endpoint will continue to function as documented.** <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch harsh event details for a vehicle.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        vehicle_id : int
            ID of the vehicle. Must contain only digits 0-9.

        timestamp : int
            Timestamp in milliseconds representing the timestamp of a harsh event.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1VehicleHarshEventResponse]
            Harsh event details.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/fleet/vehicles/{jsonable_encoder(vehicle_id)}/safety/harsh_event",
            method="GET",
            params={
                "timestamp": timestamp,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1VehicleHarshEventResponse,
                    parse_obj_as(
                        type_=V1VehicleHarshEventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLegacyApIsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_dvir_defects(
        self,
        *,
        start_time: str,
        end_time: str,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        is_resolved: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DefectsResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/streamdefects) instead. The endpoint will continue to function as documented.**

        Returns a list of DVIR defects in an organization, filtered by creation time. The maximum time period you can query for is 30 days.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Defects** under the Maintenance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). *The maximum time period you can query for is 30 days.*

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). *The maximum time period you can query for is 30 days.*

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        is_resolved : typing.Optional[bool]
            A filter on the data based on resolution status. Example: `isResolved=true`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DefectsResponse]
            List of all DVIR defects in the organization
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/defects/history",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "isResolved": is_resolved,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DefectsResponse,
                    parse_obj_as(
                        type_=DefectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_drivers_vehicle_assignments(
        self,
        *,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        driver_activation_status: typing.Optional[GetDriversVehicleAssignmentsRequestDriverActivationStatus] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getdrivervehicleassignments) instead. The endpoint will continue to function as documented.** Get all vehicle assignments for the requested drivers in the requested time range. The only type of assignment supported right now are assignments created through the driver app.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        driver_activation_status : typing.Optional[GetDriversVehicleAssignmentsRequestDriverActivationStatus]
            If value is `deactivated`, only drivers that are deactivated will appear in the response. This parameter will default to `active` if not provided (fetching only active drivers).  Valid values: `active`, `deactivated`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/drivers/vehicle-assignments",
            method="GET",
            params={
                "driverIds": driver_ids,
                "startTime": start_time,
                "endTime": end_time,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "driverActivationStatus": driver_activation_status,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody,
                    parse_obj_as(
                        type_=DriversVehicleAssignmentsGetDriversVehicleAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_dvir_history(
        self,
        *,
        start_time: str,
        end_time: str,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DvirsListResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getdvirs) instead. The endpoint will continue to function as documented.**

         Returns a list of all DVIRs in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read DVIRs** under the Maintenance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DvirsListResponse]
            List of all DVIRs in the organization
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/dvirs/history",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "startTime": start_time,
                "endTime": end_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DvirsListResponse,
                    parse_obj_as(
                        type_=DvirsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_vehicle_idling_reports(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        is_pto_active: typing.Optional[bool] = None,
        min_idling_duration_minutes: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[IdlingReportsGetVehicleIdlingReportsResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getidlingevents) instead. The endpoint will continue to function as documented.** Get all vehicle idling reports for the requested time duration.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Fuel & Energy** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        is_pto_active : typing.Optional[bool]
            A filter on the data based on power take-off being active or inactive.

        min_idling_duration_minutes : typing.Optional[int]
            A filter on the data based on a minimum idling duration.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[IdlingReportsGetVehicleIdlingReportsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/reports/vehicle/idling",
            method="GET",
            params={
                "after": after,
                "limit": limit,
                "startTime": start_time,
                "endTime": end_time,
                "vehicleIds": vehicle_ids,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "isPtoActive": is_pto_active,
                "minIdlingDurationMinutes": min_idling_duration_minutes,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    IdlingReportsGetVehicleIdlingReportsResponseBody,
                    parse_obj_as(
                        type_=IdlingReportsGetVehicleIdlingReportsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_safety_events(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SafetyEventsListResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getsafetyeventsv2stream) instead. The endpoint will continue to function as documented.**

         Fetch safety events for the organization in a given time period.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of vehicle IDs. Example: `vehicleIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SafetyEventsListResponse]
            List of safety events from given time period.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/safety-events",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "vehicleIds": vehicle_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsListResponse,
                    parse_obj_as(
                        type_=SafetyEventsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_safety_activity_event_feed(
        self,
        *,
        after: typing.Optional[str] = None,
        start_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SafetyEventsGetSafetyActivityEventFeedResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getsafetyeventsv2stream) instead. The endpoint will continue to function as documented.**

        Get continuous safety events. The safety activity event feed offers a change-log for safety events. Use this endpoint to subscribe to safety event changes. See documentation below for all supported change-log types.

        | ActivityType      | Description |
        | ----------- | ----------- |
        | CreateSafetyEventActivityType | a new safety event is processed by Samsara      |
        | BehaviorLabelActivityType     | a label is added or removed from a safety event |
        | CoachingStateActivityType     | a safety event coaching state is updated        |

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SafetyEventsGetSafetyActivityEventFeedResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/safety-events/audit-logs/feed",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SafetyEventsGetSafetyActivityEventFeedResponseBody,
                    parse_obj_as(
                        type_=SafetyEventsGetSafetyActivityEventFeedResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_vehicles_driver_assignments(
        self,
        *,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        vehicle_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getdrivervehicleassignments) instead. The endpoint will continue to function as documented.** Get all driver assignments for the requested vehicles in the requested time range. The only type of assignment supported right now are assignments created through the driver app.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00). The maximum allowed startTime-endTime range is 7 days.

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/vehicles/driver-assignments",
            method="GET",
            params={
                "startTime": start_time,
                "endTime": end_time,
                "vehicleIds": vehicle_ids,
                "tagIds": tag_ids,
                "parentTagIds": parent_tag_ids,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody,
                    parse_obj_as(
                        type_=VehiclesDriverAssignmentsGetVehiclesDriverAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_vehicle_harsh_event(
        self, vehicle_id: int, *, timestamp: int, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1VehicleHarshEventResponse]:
        """
        **Note: This is a legacy endpoint, consider using [this endpoint](https://developers.samsara.com/reference/getsafetyeventsv2stream) instead. The endpoint will continue to function as documented.** <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Fetch harsh event details for a vehicle.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Safety Events & Scores** under the Safety & Cameras category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        vehicle_id : int
            ID of the vehicle. Must contain only digits 0-9.

        timestamp : int
            Timestamp in milliseconds representing the timestamp of a harsh event.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1VehicleHarshEventResponse]
            Harsh event details.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/fleet/vehicles/{jsonable_encoder(vehicle_id)}/safety/harsh_event",
            method="GET",
            params={
                "timestamp": timestamp,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1VehicleHarshEventResponse,
                    parse_obj_as(
                        type_=V1VehicleHarshEventResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vision_step_results import V1VisionStepResults


class V1VisionRunByCameraResponseInspectionResults(UniversalBaseModel):
    capture_at_ms: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="captureAtMs"), pydantic.Field(alias="captureAtMs")
    ] = None
    result: typing.Optional[str] = None
    step_results: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResults], FieldMetadata(alias="stepResults"), pydantic.Field(alias="stepResults")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FormSubmissionRequestTableCellObjectRequestBodyType = typing.Union[
    typing.Literal["number", "text", "multiple_choice", "check_boxes", "datetime", "person"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

TimeRangeObjectRequestBodyDaysOfWeekItem = typing.Union[
    typing.Literal["FRIDAY", "MONDAY", "SATURDAY", "SUNDAY", "THURSDAY", "TUESDAY", "WEDNESDAY"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .assets_location_link_config_address_details_object import AssetsLocationLinkConfigAddressDetailsObject


class AssetsLocationLinkRequestConfigObject(UniversalBaseModel):
    """
    Configuration details specific to the 'By Asset' Live Sharing Link.
    """

    asset_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assetId"),
        pydantic.Field(alias="assetId", description="Unique assets ID that Live Sharing Link will show."),
    ] = None
    location: typing.Optional[AssetsLocationLinkConfigAddressDetailsObject] = None
    tag_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="tagIds"),
        pydantic.Field(alias="tagIds", description="Array of tag IDs to filter data by."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .missing_state_response_body_update_source import MissingStateResponseBodyUpdateSource
from .notification_recipient_response_response_body import NotificationRecipientResponseResponseBody


class MissingStateResponseBody(UniversalBaseModel):
    """
    An asset currently marked as missing, including notification recipients.
    """

    id: str = pydantic.Field()
    """
    The unique Samsara ID of the asset.
    """

    name: str = pydantic.Field()
    """
    The human-readable name of the asset.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    A note associated with the missing asset.
    """

    notification_recipients: typing.Optional[typing.List[NotificationRecipientResponseResponseBody]] = pydantic.Field(
        default=None
    )
    """
    Users subscribed to location update notifications for this asset.
    """

    update_source: typing.Optional[MissingStateResponseBodyUpdateSource] = pydantic.Field(default=None)
    """
    The source of the last update to this recovery state. Defaults to `dashboard` if not explicitly set.  Valid values: `dashboard`, `api`
    """

    updated_at_ms: int = pydantic.Field()
    """
    Timestamp when the asset was marked as missing, in milliseconds since epoch.
    """

    updated_by_user_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    The ID of the user who marked the asset as missing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..types.list_tags_response import ListTagsResponse
from ..types.tag_response import TagResponse
from ..types.tagged_object_id import TaggedObjectId

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTagsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_tags(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListTagsResponse]:
        """
        Return all of the tags for an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListTagsResponse]
            List of tags.
        """
        _response = self._client_wrapper.httpx_client.request(
            "tags",
            method="GET",
            params={
                "limit": limit,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListTagsResponse,
                    parse_obj_as(
                        type_=ListTagsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_tag(
        self,
        *,
        name: str,
        addresses: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        assets: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        drivers: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        machines: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        parent_tag_id: typing.Optional[str] = OMIT,
        sensors: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        vehicles: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TagResponse]:
        """
        Create a new tag for the organization. This may include up to 20,000 tagged entities.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        name : str
            Name of this tag.

        addresses : typing.Optional[typing.Sequence[TaggedObjectId]]
            The addresses that belong to this tag.

        assets : typing.Optional[typing.Sequence[TaggedObjectId]]
            The trailers, unpowered, and powered assets that belong to this tag.

        drivers : typing.Optional[typing.Sequence[TaggedObjectId]]
            The drivers that belong to this tag.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        machines : typing.Optional[typing.Sequence[TaggedObjectId]]
            The machines that belong to this tag.

        parent_tag_id : typing.Optional[str]
            If this tag is part a hierarchical tag tree, this is the ID of the parent tag, otherwise this will be omitted.

        sensors : typing.Optional[typing.Sequence[TaggedObjectId]]
            The sensors that belong to this tag.

        vehicles : typing.Optional[typing.Sequence[TaggedObjectId]]
            The vehicles that belong to this tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TagResponse]
            Newly created tag object, including the new tag ID.
        """
        _response = self._client_wrapper.httpx_client.request(
            "tags",
            method="POST",
            json={
                "addresses": addresses,
                "assets": assets,
                "drivers": drivers,
                "externalIds": external_ids,
                "machines": machines,
                "name": name,
                "parentTagId": parent_tag_id,
                "sensors": sensors,
                "vehicles": vehicles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_tag(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[TagResponse]:
        """
        Fetch a tag by id.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TagResponse]
            The tag corresponding to request id.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def replace_tag(
        self,
        id: str,
        *,
        addresses: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        assets: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        drivers: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        machines: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        name: typing.Optional[str] = OMIT,
        parent_tag_id: typing.Optional[str] = OMIT,
        sensors: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        vehicles: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TagResponse]:
        """
        Update a tag with a new name and new members. This API call would replace all old members of a tag with new members specified in the request body.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        addresses : typing.Optional[typing.Sequence[TaggedObjectId]]
            The addresses that belong to this tag.

        assets : typing.Optional[typing.Sequence[TaggedObjectId]]
            The trailers, unpowered, and powered assets that belong to this tag.

        drivers : typing.Optional[typing.Sequence[TaggedObjectId]]
            The drivers that belong to this tag.

        machines : typing.Optional[typing.Sequence[TaggedObjectId]]
            The machines that belong to this tag.

        name : typing.Optional[str]
            Name of this tag.

        parent_tag_id : typing.Optional[str]
            If this tag is part a hierarchical tag tree, this is the ID of the parent tag, otherwise this will be omitted.

        sensors : typing.Optional[typing.Sequence[TaggedObjectId]]
            The sensors that belong to this tag.

        vehicles : typing.Optional[typing.Sequence[TaggedObjectId]]
            The vehicles that belong to this tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TagResponse]
            The updated tag data.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "addresses": addresses,
                "assets": assets,
                "drivers": drivers,
                "machines": machines,
                "name": name,
                "parentTagId": parent_tag_id,
                "sensors": sensors,
                "vehicles": vehicles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Permanently deletes a tag.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def patch_tag(
        self,
        id: str,
        *,
        addresses: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        assets: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        drivers: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        machines: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        name: typing.Optional[str] = OMIT,
        parent_tag_id: typing.Optional[str] = OMIT,
        sensors: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        vehicles: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TagResponse]:
        """
        Update an existing tag. **Note** this implementation of patch uses [the JSON merge patch](https://tools.ietf.org/html/rfc7396) proposed standard.

         This means that any fields included in the patch request will _overwrite_ fields which exist on the target resource.

         For arrays, this means any array included in the request will _replace_ the array that exists at the specified path, it will not _add_ to the existing array.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        addresses : typing.Optional[typing.Sequence[TaggedObjectId]]
            The addresses that belong to this tag.

        assets : typing.Optional[typing.Sequence[TaggedObjectId]]
            The trailers, unpowered, and powered assets that belong to this tag.

        drivers : typing.Optional[typing.Sequence[TaggedObjectId]]
            The drivers that belong to this tag.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        machines : typing.Optional[typing.Sequence[TaggedObjectId]]
            The machines that belong to this tag.

        name : typing.Optional[str]
            Name of this tag.

        parent_tag_id : typing.Optional[str]
            If this tag is part a hierarchical tag tree, this is the ID of the parent tag, otherwise this will be omitted.

        sensors : typing.Optional[typing.Sequence[TaggedObjectId]]
            The sensors that belong to this tag.

        vehicles : typing.Optional[typing.Sequence[TaggedObjectId]]
            The vehicles that belong to this tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TagResponse]
            Returns updated tag object.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "addresses": addresses,
                "assets": assets,
                "drivers": drivers,
                "externalIds": external_ids,
                "machines": machines,
                "name": name,
                "parentTagId": parent_tag_id,
                "sensors": sensors,
                "vehicles": vehicles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTagsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_tags(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListTagsResponse]:
        """
        Return all of the tags for an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListTagsResponse]
            List of tags.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tags",
            method="GET",
            params={
                "limit": limit,
                "after": after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListTagsResponse,
                    parse_obj_as(
                        type_=ListTagsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_tag(
        self,
        *,
        name: str,
        addresses: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        assets: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        drivers: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        machines: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        parent_tag_id: typing.Optional[str] = OMIT,
        sensors: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        vehicles: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TagResponse]:
        """
        Create a new tag for the organization. This may include up to 20,000 tagged entities.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        name : str
            Name of this tag.

        addresses : typing.Optional[typing.Sequence[TaggedObjectId]]
            The addresses that belong to this tag.

        assets : typing.Optional[typing.Sequence[TaggedObjectId]]
            The trailers, unpowered, and powered assets that belong to this tag.

        drivers : typing.Optional[typing.Sequence[TaggedObjectId]]
            The drivers that belong to this tag.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        machines : typing.Optional[typing.Sequence[TaggedObjectId]]
            The machines that belong to this tag.

        parent_tag_id : typing.Optional[str]
            If this tag is part a hierarchical tag tree, this is the ID of the parent tag, otherwise this will be omitted.

        sensors : typing.Optional[typing.Sequence[TaggedObjectId]]
            The sensors that belong to this tag.

        vehicles : typing.Optional[typing.Sequence[TaggedObjectId]]
            The vehicles that belong to this tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TagResponse]
            Newly created tag object, including the new tag ID.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tags",
            method="POST",
            json={
                "addresses": addresses,
                "assets": assets,
                "drivers": drivers,
                "externalIds": external_ids,
                "machines": machines,
                "name": name,
                "parentTagId": parent_tag_id,
                "sensors": sensors,
                "vehicles": vehicles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_tag(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TagResponse]:
        """
        Fetch a tag by id.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TagResponse]
            The tag corresponding to request id.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def replace_tag(
        self,
        id: str,
        *,
        addresses: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        assets: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        drivers: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        machines: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        name: typing.Optional[str] = OMIT,
        parent_tag_id: typing.Optional[str] = OMIT,
        sensors: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        vehicles: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TagResponse]:
        """
        Update a tag with a new name and new members. This API call would replace all old members of a tag with new members specified in the request body.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        addresses : typing.Optional[typing.Sequence[TaggedObjectId]]
            The addresses that belong to this tag.

        assets : typing.Optional[typing.Sequence[TaggedObjectId]]
            The trailers, unpowered, and powered assets that belong to this tag.

        drivers : typing.Optional[typing.Sequence[TaggedObjectId]]
            The drivers that belong to this tag.

        machines : typing.Optional[typing.Sequence[TaggedObjectId]]
            The machines that belong to this tag.

        name : typing.Optional[str]
            Name of this tag.

        parent_tag_id : typing.Optional[str]
            If this tag is part a hierarchical tag tree, this is the ID of the parent tag, otherwise this will be omitted.

        sensors : typing.Optional[typing.Sequence[TaggedObjectId]]
            The sensors that belong to this tag.

        vehicles : typing.Optional[typing.Sequence[TaggedObjectId]]
            The vehicles that belong to this tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TagResponse]
            The updated tag data.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "addresses": addresses,
                "assets": assets,
                "drivers": drivers,
                "machines": machines,
                "name": name,
                "parentTagId": parent_tag_id,
                "sensors": sensors,
                "vehicles": vehicles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Permanently deletes a tag.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def patch_tag(
        self,
        id: str,
        *,
        addresses: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        assets: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        drivers: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        machines: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        name: typing.Optional[str] = OMIT,
        parent_tag_id: typing.Optional[str] = OMIT,
        sensors: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        vehicles: typing.Optional[typing.Sequence[TaggedObjectId]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TagResponse]:
        """
        Update an existing tag. **Note** this implementation of patch uses [the JSON merge patch](https://tools.ietf.org/html/rfc7396) proposed standard.

         This means that any fields included in the patch request will _overwrite_ fields which exist on the target resource.

         For arrays, this means any array included in the request will _replace_ the array that exists at the specified path, it will not _add_ to the existing array.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Tags** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Tag. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.name:ELD-exempt`.

        addresses : typing.Optional[typing.Sequence[TaggedObjectId]]
            The addresses that belong to this tag.

        assets : typing.Optional[typing.Sequence[TaggedObjectId]]
            The trailers, unpowered, and powered assets that belong to this tag.

        drivers : typing.Optional[typing.Sequence[TaggedObjectId]]
            The drivers that belong to this tag.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        machines : typing.Optional[typing.Sequence[TaggedObjectId]]
            The machines that belong to this tag.

        name : typing.Optional[str]
            Name of this tag.

        parent_tag_id : typing.Optional[str]
            If this tag is part a hierarchical tag tree, this is the ID of the parent tag, otherwise this will be omitted.

        sensors : typing.Optional[typing.Sequence[TaggedObjectId]]
            The sensors that belong to this tag.

        vehicles : typing.Optional[typing.Sequence[TaggedObjectId]]
            The vehicles that belong to this tag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TagResponse]
            Returns updated tag object.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tags/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "addresses": addresses,
                "assets": assets,
                "drivers": drivers,
                "externalIds": external_ids,
                "machines": machines,
                "name": name,
                "parentTagId": parent_tag_id,
                "sensors": sensors,
                "vehicles": vehicles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TagResponse,
                    parse_obj_as(
                        type_=TagResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .reverse_geo import ReverseGeo
from .time import Time
from .vehicle_location_heading import VehicleLocationHeading
from .vehicle_location_latitude import VehicleLocationLatitude
from .vehicle_location_longitude import VehicleLocationLongitude
from .vehicle_location_speed import VehicleLocationSpeed


class VehicleLocation(UniversalBaseModel):
    """
    Vehicle location event.
    """

    heading: typing.Optional[VehicleLocationHeading] = None
    latitude: VehicleLocationLatitude
    longitude: VehicleLocationLongitude
    reverse_geo: typing_extensions.Annotated[
        typing.Optional[ReverseGeo], FieldMetadata(alias="reverseGeo"), pydantic.Field(alias="reverseGeo")
    ] = None
    speed: typing.Optional[VehicleLocationSpeed] = None
    time: Time

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .asset_data_output import AssetDataOutput
from .asset_location import AssetLocation
from .asset_name import AssetName
from .asset_response_location_data_input import AssetResponseLocationDataInput
from .asset_response_parent_asset import AssetResponseParentAsset
from .asset_response_running_status_data_input import AssetResponseRunningStatusDataInput
from .custom_metadata import CustomMetadata
from .location_type import LocationType
from .tag_tiny_response import TagTinyResponse


class AssetResponse(UniversalBaseModel):
    """
    Asset
    """

    custom_metadata: typing_extensions.Annotated[
        typing.Optional[CustomMetadata], FieldMetadata(alias="customMetadata"), pydantic.Field(alias="customMetadata")
    ] = None
    data_outputs: typing_extensions.Annotated[
        typing.Optional[typing.List[AssetDataOutput]],
        FieldMetadata(alias="dataOutputs"),
        pydantic.Field(alias="dataOutputs", description="The list of data outputs configured on the asset."),
    ] = None
    id: str = pydantic.Field()
    """
    The id of the asset
    """

    is_running: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isRunning"),
        pydantic.Field(
            alias="isRunning", description="The running status of the asset. Returns True for On, and False for Off."
        ),
    ]
    location: typing.Optional[AssetLocation] = None
    location_data_input: typing_extensions.Annotated[
        typing.Optional[AssetResponseLocationDataInput],
        FieldMetadata(alias="locationDataInput"),
        pydantic.Field(alias="locationDataInput"),
    ] = None
    location_type: typing_extensions.Annotated[
        typing.Optional[LocationType], FieldMetadata(alias="locationType"), pydantic.Field(alias="locationType")
    ] = None
    name: AssetName
    parent_asset: typing_extensions.Annotated[
        typing.Optional[AssetResponseParentAsset],
        FieldMetadata(alias="parentAsset"),
        pydantic.Field(alias="parentAsset"),
    ] = None
    running_status_data_input: typing_extensions.Annotated[
        typing.Optional[AssetResponseRunningStatusDataInput],
        FieldMetadata(alias="runningStatusDataInput"),
        pydantic.Field(alias="runningStatusDataInput"),
    ] = None
    tags: typing.Optional[typing.List[TagTinyResponse]] = pydantic.Field(default=None)
    """
    The list of [tags](https://kb.samsara.com/hc/en-us/articles/360026674631-Using-Tags-and-Tag-Nesting) associated with the Industrial Asset. **By default**: empty. Can be set or updated through the Samsara Dashboard or the API at any time.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ambient_temperature_details_object_response_body import AmbientTemperatureDetailsObjectResponseBody
from .cell_signal_loss_details_object_response_body import CellSignalLossDetailsObjectResponseBody
from .def_level_trigger_details_object_response_body import DefLevelTriggerDetailsObjectResponseBody
from .device_movement_trigger_details_object_response_body import DeviceMovementTriggerDetailsObjectResponseBody
from .driver_document_submitted_details_object_response_body import DriverDocumentSubmittedDetailsObjectResponseBody
from .dvir_submitted_device_trigger_details_object_response_body import (
    DvirSubmittedDeviceTriggerDetailsObjectResponseBody,
)
from .engine_idle_trigger_details_object_response_body import EngineIdleTriggerDetailsObjectResponseBody
from .engine_off_details_object_response_body import EngineOffDetailsObjectResponseBody
from .engine_on_details_object_response_body import EngineOnDetailsObjectResponseBody
from .fuel_level_trigger_details_object_response_body import FuelLevelTriggerDetailsObjectResponseBody
from .gateway_disconnected_details_object_response_body import GatewayDisconnectedDetailsObjectResponseBody
from .gateway_unplugged_trigger_details_object_response_body import GatewayUnpluggedTriggerDetailsObjectResponseBody
from .geofence_entry_trigger_details_object_response_body import GeofenceEntryTriggerDetailsObjectResponseBody
from .geofence_exit_trigger_details_object_response_body import GeofenceExitTriggerDetailsObjectResponseBody
from .gps_signal_loss_details_object_response_body import GpsSignalLossDetailsObjectResponseBody
from .harsh_event_trigger_details_object_response_body import HarshEventTriggerDetailsObjectResponseBody
from .hos_violation_trigger_details_object_response_body import HosViolationTriggerDetailsObjectResponseBody
from .inside_geofence_trigger_details_object_response_body import InsideGeofenceTriggerDetailsObjectResponseBody
from .out_of_route_details_object_response_body import OutOfRouteDetailsObjectResponseBody
from .outside_geofence_trigger_details_object_response_body import OutsideGeofenceTriggerDetailsObjectResponseBody
from .panic_button_details_object_response_body import PanicButtonDetailsObjectResponseBody
from .reading_trigger_details_object_response_body import ReadingTriggerDetailsObjectResponseBody
from .route_stop_estimated_arrival_details_object_response_body import (
    RouteStopEstimatedArrivalDetailsObjectResponseBody,
)
from .safety_behavior_trigger_details_object_response_body import SafetyBehaviorTriggerDetailsObjectResponseBody
from .scheduled_maintenance_by_engine_hours_details_object_response_body import (
    ScheduledMaintenanceByEngineHoursDetailsObjectResponseBody,
)
from .scheduled_maintenance_odometer_trigger_details_object_response_body import (
    ScheduledMaintenanceOdometerTriggerDetailsObjectResponseBody,
)
from .scheduled_maintenance_trigger_details_object_response_body import (
    ScheduledMaintenanceTriggerDetailsObjectResponseBody,
)
from .speed_trigger_details_object_response_body import SpeedTriggerDetailsObjectResponseBody
from .sudden_fuel_level_drop_trigger_details_object_response_body import (
    SuddenFuelLevelDropTriggerDetailsObjectResponseBody,
)
from .sudden_fuel_level_rise_trigger_details_object_response_body import (
    SuddenFuelLevelRiseTriggerDetailsObjectResponseBody,
)
from .tire_fault_code_details_object_response_body import TireFaultCodeDetailsObjectResponseBody
from .training_assignment_near_due_date_trigger_details_object_response_body import (
    TrainingAssignmentNearDueDateTriggerDetailsObjectResponseBody,
)
from .unassigned_driving_trigger_details_object_response_body import UnassignedDrivingTriggerDetailsObjectResponseBody
from .vehicle_batter_voltage_details_object_response_body import VehicleBatterVoltageDetailsObjectResponseBody
from .vehicle_fault_code_details_object_response_body import VehicleFaultCodeDetailsObjectResponseBody


class TriggerParamsObjectResponseBody(UniversalBaseModel):
    """
    The trigger type specific details. Only the field that corresponds to the trigger type is filled in.
    """

    ambient_temperature: typing_extensions.Annotated[
        typing.Optional[AmbientTemperatureDetailsObjectResponseBody],
        FieldMetadata(alias="ambientTemperature"),
        pydantic.Field(alias="ambientTemperature"),
    ] = None
    cell_signal_loss: typing_extensions.Annotated[
        typing.Optional[CellSignalLossDetailsObjectResponseBody],
        FieldMetadata(alias="cellSignalLoss"),
        pydantic.Field(alias="cellSignalLoss"),
    ] = None
    def_level: typing_extensions.Annotated[
        typing.Optional[DefLevelTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="defLevel"),
        pydantic.Field(alias="defLevel"),
    ] = None
    device_movement: typing_extensions.Annotated[
        typing.Optional[DeviceMovementTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="deviceMovement"),
        pydantic.Field(alias="deviceMovement"),
    ] = None
    document_submitted: typing_extensions.Annotated[
        typing.Optional[DriverDocumentSubmittedDetailsObjectResponseBody],
        FieldMetadata(alias="documentSubmitted"),
        pydantic.Field(alias="documentSubmitted"),
    ] = None
    dvir_submitted_device: typing_extensions.Annotated[
        typing.Optional[DvirSubmittedDeviceTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="dvirSubmittedDevice"),
        pydantic.Field(alias="dvirSubmittedDevice"),
    ] = None
    engine_idle: typing_extensions.Annotated[
        typing.Optional[EngineIdleTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="engineIdle"),
        pydantic.Field(alias="engineIdle"),
    ] = None
    engine_off: typing_extensions.Annotated[
        typing.Optional[EngineOffDetailsObjectResponseBody],
        FieldMetadata(alias="engineOff"),
        pydantic.Field(alias="engineOff"),
    ] = None
    engine_on: typing_extensions.Annotated[
        typing.Optional[EngineOnDetailsObjectResponseBody],
        FieldMetadata(alias="engineOn"),
        pydantic.Field(alias="engineOn"),
    ] = None
    fuel_level: typing_extensions.Annotated[
        typing.Optional[FuelLevelTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="fuelLevel"),
        pydantic.Field(alias="fuelLevel"),
    ] = None
    gateway_disconnected: typing_extensions.Annotated[
        typing.Optional[GatewayDisconnectedDetailsObjectResponseBody],
        FieldMetadata(alias="gatewayDisconnected"),
        pydantic.Field(alias="gatewayDisconnected"),
    ] = None
    gateway_unplugged: typing_extensions.Annotated[
        typing.Optional[GatewayUnpluggedTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="gatewayUnplugged"),
        pydantic.Field(alias="gatewayUnplugged"),
    ] = None
    geofence_entry: typing_extensions.Annotated[
        typing.Optional[GeofenceEntryTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="geofenceEntry"),
        pydantic.Field(alias="geofenceEntry"),
    ] = None
    geofence_exit: typing_extensions.Annotated[
        typing.Optional[GeofenceExitTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="geofenceExit"),
        pydantic.Field(alias="geofenceExit"),
    ] = None
    gps_signal_loss: typing_extensions.Annotated[
        typing.Optional[GpsSignalLossDetailsObjectResponseBody],
        FieldMetadata(alias="gpsSignalLoss"),
        pydantic.Field(alias="gpsSignalLoss"),
    ] = None
    harsh_event: typing_extensions.Annotated[
        typing.Optional[HarshEventTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="harshEvent"),
        pydantic.Field(alias="harshEvent"),
    ] = None
    hos_violation: typing_extensions.Annotated[
        typing.Optional[HosViolationTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="hosViolation"),
        pydantic.Field(alias="hosViolation"),
    ] = None
    inside_geofence: typing_extensions.Annotated[
        typing.Optional[InsideGeofenceTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="insideGeofence"),
        pydantic.Field(alias="insideGeofence"),
    ] = None
    out_of_route: typing_extensions.Annotated[
        typing.Optional[OutOfRouteDetailsObjectResponseBody],
        FieldMetadata(alias="outOfRoute"),
        pydantic.Field(alias="outOfRoute"),
    ] = None
    outside_geofence: typing_extensions.Annotated[
        typing.Optional[OutsideGeofenceTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="outsideGeofence"),
        pydantic.Field(alias="outsideGeofence"),
    ] = None
    panic_button: typing_extensions.Annotated[
        typing.Optional[PanicButtonDetailsObjectResponseBody],
        FieldMetadata(alias="panicButton"),
        pydantic.Field(alias="panicButton"),
    ] = None
    reading: typing.Optional[ReadingTriggerDetailsObjectResponseBody] = None
    route_stop_estimated_arrival: typing_extensions.Annotated[
        typing.Optional[RouteStopEstimatedArrivalDetailsObjectResponseBody],
        FieldMetadata(alias="routeStopEstimatedArrival"),
        pydantic.Field(alias="routeStopEstimatedArrival"),
    ] = None
    safety_behavior: typing_extensions.Annotated[
        typing.Optional[SafetyBehaviorTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="safetyBehavior"),
        pydantic.Field(alias="safetyBehavior"),
    ] = None
    scheduled_maintenance: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="scheduledMaintenance"),
        pydantic.Field(alias="scheduledMaintenance"),
    ] = None
    scheduled_maintenance_by_engine_hours: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceByEngineHoursDetailsObjectResponseBody],
        FieldMetadata(alias="scheduledMaintenanceByEngineHours"),
        pydantic.Field(alias="scheduledMaintenanceByEngineHours"),
    ] = None
    scheduled_maintenance_odometer: typing_extensions.Annotated[
        typing.Optional[ScheduledMaintenanceOdometerTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="scheduledMaintenanceOdometer"),
        pydantic.Field(alias="scheduledMaintenanceOdometer"),
    ] = None
    speed: typing.Optional[SpeedTriggerDetailsObjectResponseBody] = None
    sudden_fuel_level_drop: typing_extensions.Annotated[
        typing.Optional[SuddenFuelLevelDropTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="suddenFuelLevelDrop"),
        pydantic.Field(alias="suddenFuelLevelDrop"),
    ] = None
    sudden_fuel_level_rise: typing_extensions.Annotated[
        typing.Optional[SuddenFuelLevelRiseTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="suddenFuelLevelRise"),
        pydantic.Field(alias="suddenFuelLevelRise"),
    ] = None
    tire_fault_code: typing_extensions.Annotated[
        typing.Optional[TireFaultCodeDetailsObjectResponseBody],
        FieldMetadata(alias="tireFaultCode"),
        pydantic.Field(alias="tireFaultCode"),
    ] = None
    training_assignment_near_due_date: typing_extensions.Annotated[
        typing.Optional[TrainingAssignmentNearDueDateTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="trainingAssignmentNearDueDate"),
        pydantic.Field(alias="trainingAssignmentNearDueDate"),
    ] = None
    unassigned_driving: typing_extensions.Annotated[
        typing.Optional[UnassignedDrivingTriggerDetailsObjectResponseBody],
        FieldMetadata(alias="unassignedDriving"),
        pydantic.Field(alias="unassignedDriving"),
    ] = None
    vehicle_battery_voltage: typing_extensions.Annotated[
        typing.Optional[VehicleBatterVoltageDetailsObjectResponseBody],
        FieldMetadata(alias="vehicleBatteryVoltage"),
        pydantic.Field(alias="vehicleBatteryVoltage"),
    ] = None
    vehicle_fault_code: typing_extensions.Annotated[
        typing.Optional[VehicleFaultCodeDetailsObjectResponseBody],
        FieldMetadata(alias="vehicleFaultCode"),
        pydantic.Field(alias="vehicleFaultCode"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .safety_score_behavior_object_response_body_behavior_type import SafetyScoreBehaviorObjectResponseBodyBehaviorType


class SafetyScoreBehaviorObjectResponseBody(UniversalBaseModel):
    """
    Safety score behavior object.
    """

    behavior_type: typing_extensions.Annotated[
        SafetyScoreBehaviorObjectResponseBodyBehaviorType,
        FieldMetadata(alias="behaviorType"),
        pydantic.Field(
            alias="behaviorType",
            description="Type of the behavior.  Valid values: `acceleration`, `braking`, `crash`, `defensiveDriving`, `didNotYield`, `distractedDrivingAutomatic`, `distractedDrivingManual`, `drowsy`, `eatingDrinking`, `followingDistance`, `followingDistanceModerate`, `followingDistanceSevere`, `forwardCollisionWarning`, `harshTurn`, `laneDeparture`, `lateResponse`, `mobileUsage`, `nearCollision`, `noSeatbelt`, `obstructedCamera`, `ranRedLight`, `rollingStop`, `smoking`, `speedingManual`, `unknown`",
        ),
    ]
    count: int = pydantic.Field()
    """
    Count of occurrences of the behavior.
    """

    score_impact: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="scoreImpact"),
        pydantic.Field(
            alias="scoreImpact",
            description="Total points increased or deducted from the score due to the total count of behaviors of this type. Weights for this calculation are defined in organisation settings. Negative numbers indicate points deducted and positive indicates points increased (e.g. defensive driving).",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .cumulative_operating_hours_response_body import CumulativeOperatingHoursResponseBody
from .def_remaining_response_body import DefRemainingResponseBody
from .distance_response_body import DistanceResponseBody
from .engine_status_response_body import EngineStatusResponseBody
from .equipment_header_with_additional_fields_response_body import EquipmentHeaderWithAdditionalFieldsResponseBody
from .fuel_remaining_response_body import FuelRemainingResponseBody
from .location_response_body import LocationResponseBody


class AempEquipmentWithAdditionalFieldsResponseBody(UniversalBaseModel):
    """
    Contains equipment fields.
    """

    cumulative_operating_hours: typing_extensions.Annotated[
        typing.Optional[CumulativeOperatingHoursResponseBody],
        FieldMetadata(alias="CumulativeOperatingHours"),
        pydantic.Field(alias="CumulativeOperatingHours"),
    ] = None
    def_remaining: typing_extensions.Annotated[
        typing.Optional[DefRemainingResponseBody],
        FieldMetadata(alias="DEFRemaining"),
        pydantic.Field(alias="DEFRemaining"),
    ] = None
    distance: typing_extensions.Annotated[
        typing.Optional[DistanceResponseBody], FieldMetadata(alias="Distance"), pydantic.Field(alias="Distance")
    ] = None
    engine_status: typing_extensions.Annotated[
        typing.Optional[EngineStatusResponseBody],
        FieldMetadata(alias="EngineStatus"),
        pydantic.Field(alias="EngineStatus"),
    ] = None
    equipment_header: typing_extensions.Annotated[
        EquipmentHeaderWithAdditionalFieldsResponseBody,
        FieldMetadata(alias="EquipmentHeader"),
        pydantic.Field(alias="EquipmentHeader"),
    ]
    fuel_remaining: typing_extensions.Annotated[
        typing.Optional[FuelRemainingResponseBody],
        FieldMetadata(alias="FuelRemaining"),
        pydantic.Field(alias="FuelRemaining"),
    ] = None
    location: typing_extensions.Annotated[
        LocationResponseBody, FieldMetadata(alias="Location"), pydantic.Field(alias="Location")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

UsDriverRulesetOverrideUsStateToOverride = typing.Union[
    typing.Literal["", "AK", "CA", "FL", "NE", "NC", "OK", "OR", "SC", "TX", "WI"], typing.Any
]
